from horizon_tc_ui import HB_ONNXRuntime
import torch
import torchvision.transforms as transforms
import cv2
import numpy as np
import os
from PIL import Image 

MODEL_PATH = "./model_output/yolov5s_640x640_nv12_quantized_model.onnx"

def bgr2nv12(image): 
    image = image.astype(np.uint8) 
    height, width = image.shape[0], image.shape[1] 
    yuv420p = cv2.cvtColor(image, cv2.COLOR_BGR2YUV_I420).reshape((height * width * 3 // 2, )) 
    y = yuv420p[:height * width] 
    uv_planar = yuv420p[height * width:].reshape((2, height * width // 4)) 
    uv_packed = uv_planar.transpose((1, 0)).reshape((height * width // 2, )) 
    nv12 = np.zeros_like(yuv420p) 
    nv12[:height * width] = y 
    nv12[height * width:] = uv_packed 
    return nv12 
 
def nv12Toyuv444(nv12, target_size): 
    height = target_size[0] 
    width = target_size[1] 
    nv12_data = nv12.flatten() 
    yuv444 = np.empty([height, width, 3], dtype=np.uint8) 
    yuv444[:, :, 0] = nv12_data[:width * height].reshape(height, width) 
    u = nv12_data[width * height::2].reshape(height // 2, width // 2) 
    yuv444[:, :, 1] = Image.fromarray(u).resize((width, height),resample=0) 
    v = nv12_data[width * height + 1::2].reshape(height // 2, width // 2) 
    yuv444[:, :, 2] = Image.fromarray(v).resize((width, height),resample=0) 
    return yuv444

def preprocess(img):
    new_shape = (640, 640)

    radio = max(img.shape[0] / new_shape[0], img.shape[1] / new_shape[1])
    resize_shape = (int(img.shape[1] / radio), int(img.shape[0] / radio))
    img = cv2.resize(img, resize_shape, interpolation=cv2.INTER_LINEAR)
    top, left = 0, 0
    bottom, right = new_shape[1] - resize_shape[1], new_shape[0] - resize_shape[0]
    img = cv2.copyMakeBorder(img, top, bottom, left, right, cv2.BORDER_CONSTANT, value=(0,0,0))

    nv12 = bgr2nv12(img)
    img = nv12Toyuv444(nv12, new_shape)
    cv2.imwrite("yuv.jpg", img)

    # img = img.transpose((2, 0, 1))[::-1]  # HWC to CHW, BGR to RGB
    img = np.ascontiguousarray(img)
    # img = torch.from_numpy(img).to("cpu")
    img = img.astype(np.uint8)
    # img /= 255  # 0 - 255 to 0.0 - 1.0
    if len(img.shape) == 3:
        img = img[None]  # expand for batch dim
    print(img.shape)

    return img

def postprocess(img, tensors):
    score_threshold = 0.05
    nms_threshold = 0.5

    print(len(tensors))
    tensor = tensors[0]
    print(tensor.shape)

    boxs = []
    scores = []

    for i in range(len(tensor[0])):
        score_ori = tensor[0][i][4]
        if score_ori < score_threshold:
            continue

        # print("{:.2f}".format(score_ori))
        for j in range(32):
            score = tensor[0][i][4] * tensor[0][i][5 + j]
            # print("{:.2f} = {:.2f} * {:.2f}".format(score, ))
            if score < score_threshold:
                continue

            x1 = tensor[0][i][0] - tensor[0][i][2] / 2
            y1 = tensor[0][i][1] - tensor[0][i][3] / 2
            x2 = tensor[0][i][0] + tensor[0][i][2] / 2
            y2 = tensor[0][i][1] + tensor[0][i][3] / 2

            boxs.append((x1, y1, x2, y2))
            scores.append(score)

    print(len(boxs), len(scores))
    ids = cv2.dnn.NMSBoxes(boxs, scores, score_threshold, nms_threshold)

    new_shape = (640, 640)

    radio = max(img.shape[0] / new_shape[0], img.shape[1] / new_shape[1])

    for id in ids:
        x1, y1, x2, y2 = boxs[id]
        x1 = x1 * radio
        y1 = y1 * radio
        x2 = x2 * radio
        y2 = y2 * radio

        cv2.rectangle(img, (int(x1), int(y1)), (int(x2), int(y2)), (0, 0, 255), 1, cv2.LINE_8)
        cv2.putText(img, "{}".format(id), (int(x1), int(y1)), cv2.FONT_HERSHEY_SCRIPT_COMPLEX, 0.8, (0, 255, 0), 1)

    cv2.imwrite("output.jpg", img)

def main():
    # 加载模型⽂件
    sess = HB_ONNXRuntime(model_file=MODEL_PATH)
    # 获取输⼊&输出节点名称
    input_names = [input.name for input in sess.get_inputs()]
    output_names = [output.name for output in sess.get_outputs()]
    print("input number: {}, output number: {}".format(len(input_names), len(output_names)))

    img = cv2.imread("./test.jpg")

    # 准备模型输⼊数据
    feed_dict = dict()
    for input_name in input_names:
        feed_dict[input_name] = preprocess(img)
    # 图像输⼊的模型（RGB/BGR/NV12/YUV444/GRAY），数据dtype= uint8
    outputs = sess.run(output_names, feed_dict, input_offset=128)
    # # featuremap模型，数据dtype=float32
    # outputs = sess.run_feature(output_names, feed_dict, input_offset=0)
    # 后处理
    postprocess(img, outputs)

if __name__ == '__main__':
    main()
